from typing import Any, Optional, IO, Iterator
from argh import ArghParser
from argh.dispatching import dispatch as _dispatch
import io

from scale_egp.cli.formatter import AVAILABLE_FORMATTERS, Formatter


# based on argh.dispatching._process_command_output
def get_patched_process_command_output(formatter: Formatter):
    def patched_process_command_output(
        lines: Iterator[Any], output_file: Optional[IO], raw_output: bool
    ) -> Optional[str]:
        out_io: IO

        if output_file is None:
            # user wants a string; we create an internal temporary file-like object
            # and will return its contents as a string
            out_io = io.StringIO()
        else:
            # normally this is stdout; can be any file
            out_io = output_file

        formatter.format(out_io, lines, raw_output)

        if output_file is None:
            # user wanted a string; return contents of our temporary file-like obj
            out_io.seek(0)
            return out_io.read()

        return None

    return patched_process_command_output


def get_parser() -> ArghParser:
    return ArghParser(
        description="scale-egp - CLI for Scale EGP\nFor more information on EGP, please visit: https://scale-egp.readme.io/"
    )


def dispatch(parser: ArghParser, args: Any) -> Optional[str]:
    from mock import patch

    formatter = AVAILABLE_FORMATTERS[args.format](args)

    with patch(
        "argh.dispatching._process_command_output",
        side_effect=get_patched_process_command_output(formatter),
    ):
        return _dispatch(parser=parser, add_help_command=True)



from typing import List, Optional

import httpx
from scale_egp.sdk.models.model_schemas.parameter_schema import ParameterSchema
from scale_egp.utils.model_utils import dict_without_none_values, make_partial_model
from scale_egp.sdk.models.model_schemas.model_vendor_configuration import LaunchVendorConfiguration

from scale_egp.sdk.models.model_api_models import (
    ModelEndpointType,
    ModelTemplateRequest,
    ModelTemplate,
    ModelType,
)
from scale_egp.utils.api_utils import APIEngine, long_timeout


PartialModelTemplateRequest = make_partial_model(ModelTemplateRequest)


class ModelTemplateCollection(APIEngine):
    """
    Collections class for EGP Models.
    """

    _sub_path = "v3/model-templates"

    def create(
        self,
        name: str,
        endpoint_type: ModelEndpointType,
        model_type: ModelType,
        vendor_configuration: LaunchVendorConfiguration,
        model_creation_parameters_schema: Optional[ParameterSchema] = None,
        model_request_parameters_schema: Optional[ParameterSchema] = None,
        account_id: Optional[str] = None,
    ) -> ModelTemplate:
        """
        Create a new EGP Model.
        """
        response = self._post(
            sub_path=self._sub_path,
            request=ModelTemplateRequest(
                name=name,
                endpoint_type=endpoint_type,
                model_type=model_type,
                vendor_configuration=vendor_configuration,
                model_creation_parameters_schema=model_creation_parameters_schema,
                model_request_parameters_schema=model_request_parameters_schema,
                account_id=account_id or self._api_client.account_id,
            ),
            timeout=long_timeout,
        )
        return ModelTemplate.from_dict(response.json())

    def get(
        self,
        id: str,
    ) -> ModelTemplate:
        """
        Get a Model by ID.
        """
        response = self._get(
            sub_path=f"{self._sub_path}/{id}",
        )
        return ModelTemplate.from_dict(response.json())

    def update(
        self,
        id: str,
        *,
        name: str,
        endpoint_type: Optional[ModelEndpointType] = None,
        model_type: Optional[ModelType] = None,
        vendor_configuration: Optional[LaunchVendorConfiguration] = None,
        model_creation_parameters_schema: Optional[ParameterSchema] = None,
        model_request_parameters_schema: Optional[ParameterSchema] = None,
    ) -> ModelTemplate:
        """
        Update a Model by ID.

        """
        response = self._patch(
            sub_path=f"{self._sub_path}/{id}",
            request=PartialModelTemplateRequest(
                **dict_without_none_values(
                    dict(
                        name=name,
                        endpoint_type=endpoint_type,
                        model_type=model_type,
                        vendor_configuration=vendor_configuration,
                        model_creation_parameters_schema=model_creation_parameters_schema,
                        model_request_parameters_schema=model_request_parameters_schema,
                    )
                )
            ),
        )
        return ModelTemplate.from_dict(response.json())

    def delete(
        self,
        id: str,
    ) -> bool:
        """
        Delete a Studio Project by ID.

        """
        response = self._delete(
            sub_path=f"{self._sub_path}/{id}",
        )
        return response.status_code == httpx.codes.ok

    def list(
        self,
    ) -> List[ModelTemplate]:
        """
        List all Studio Projects.

        Returns:
            A list of Studio Projects.
        """
        response = self._get(
            sub_path=self._sub_path,
        )
        return [ModelTemplate.from_dict(model_template) for model_template in response.json()]



from typing import Optional, Literal, Iterable

from scale_egp.sdk.models import Completion, CompletionRequest, ModelParameters
from scale_egp.utils.api_utils import APIEngine


class CompletionCollection(APIEngine):

    _sub_path = "v2/completions"

    def create(
        self,
        model: Literal[
            "gpt-4",
            "gpt-4-0613",
            "gpt-4-32k",
            "gpt-4-32k-0613",
            "gpt-3.5-turbo",
            "gpt-3.5-turbo-0613",
            "gpt-3.5-turbo-16k",
            "gpt-3.5-turbo-16k-0613",
            "text-davinci-003",
            "text-davinci-002",
            "text-curie-001",
            "text-babbage-001",
            "text-ada-001",
        ],
        prompt: str,
        model_parameters: Optional[ModelParameters] = None,
    ) -> Completion:
        """
        Create a new LLM Completion.

        Args:
            model: The model to use for the completion.
            prompt: The prompt to use for the completion.
            model_parameters: The parameters to use for the model.

        Returns:
            The newly created Completion.
        """
        response = self._post(
            sub_path=self._sub_path,
            request=CompletionRequest(
                model=model,
                prompt=prompt,
                model_parameters=model_parameters,
            ),
        )
        return Completion.from_dict(response.json())

    def stream(
        self,
        model: Literal[
            "gpt-4",
            "gpt-4-0613",
            "gpt-4-32k",
            "gpt-4-32k-0613",
            "gpt-3.5-turbo",
            "gpt-3.5-turbo-0613",
            "gpt-3.5-turbo-16k",
            "gpt-3.5-turbo-16k-0613",
            "text-davinci-003",
            "text-davinci-002",
            "text-curie-001",
            "text-babbage-001",
            "text-ada-001",
        ],
        prompt: str,
        model_parameters: Optional[ModelParameters] = None,
    ) -> Iterable[Completion]:
        """
        Stream LLM Completions.

        Returns:
            The newly created Completion.
        """
        iterable_payloads = self._post_stream(
            sub_path=self._sub_path,
            request=CompletionRequest(
                model=model,
                prompt=prompt,
                model_parameters=model_parameters,
                stream=True,
            ),
        )
        if iterable_payloads:
            for response_dict in iterable_payloads:
                yield Completion.from_dict(response_dict)
        else:
            return []



# !!!!! DO NOT EVER CHANGE THIS FILE MANUALLY -- AUTOGENERATED by orm2pydantic.py !!!!!
# Generated on 2023-12-23 from module egp_api_backend.server.internal.orm.orm_models in the egp-api-backend package for use in the EGP python SDK
# To regenerate this file, run:
# scaleapi/packages/egp-api-backend/scripts/orm2pydantic.sh

from datetime import datetime
from typing import Any, Dict, Optional

from pydantic import Field
from scale_egp.sdk.models.model_schemas.model_enums import ModelEndpointType, ModelType, ModelVendor
from scale_egp.sdk.models.model_schemas.model_vendor_configuration import LaunchVendorConfiguration
from scale_egp.sdk.models.model_schemas.parameter_schema import ParameterSchema
from scale_egp.utils.model_utils import BaseModel, Entity


class ModelTemplateRequest(BaseModel):
    name: str
    endpoint_type: ModelEndpointType
    model_type: ModelType
    vendor_configuration: LaunchVendorConfiguration
    model_creation_parameters_schema: Optional[ParameterSchema] = Field(
        None,
        description="The field names and types of available parameter fields which may be specified during model creation",
    )
    model_request_parameters_schema: Optional[ParameterSchema] = Field(
        None,
        description="The field names and types of available parameter fields which may be specified in a model execution API's `model_request_parameters` field.",
    )
    account_id: str = Field(..., description="The ID of the account that owns the given entity.")


class ModelTemplate(Entity):
    """
    This is a template for types of models that can be quickly customized by end users.
    It allows users to upload static docker images that can run specific types of models.
    These docker images will expose parameters that can be injected at ModelAlias creation
    time to customize the functionality. A common example of this is to use a
    HuggingFace LLM template, but swap out model weights for a finetuned model.
    """

    name: str
    endpoint_type: ModelEndpointType
    model_type: ModelType
    vendor_configuration: LaunchVendorConfiguration
    model_creation_parameters_schema: Optional[ParameterSchema] = Field(
        None,
        description="The field names and types of available parameter fields which may be specified during model creation",
    )
    model_request_parameters_schema: Optional[ParameterSchema] = Field(
        None,
        description="The field names and types of available parameter fields which may be specified in a model execution API's `model_request_parameters` field.",
    )
    id: str = Field(..., description="The unique identifier of the entity.")
    created_at: datetime = Field(
        ..., description="The date and time when the entity was created in ISO format."
    )
    account_id: str = Field(..., description="The ID of the account that owns the given entity.")
    created_by_user_id: str = Field(..., description="The user who originally created the entity.")


class ModelEndpointRequest(BaseModel):
    name: str


class ModelEndpoint(Entity):
    name: str
    status: str
    id: str = Field(..., description="The unique identifier of the entity.")
    created_at: datetime = Field(
        ..., description="The date and time when the entity was created in ISO format."
    )
    created_by_user_id: str = Field(..., description="The user who originally created the entity.")


class ModelAliasRequest(BaseModel):
    name: str
    model_vendor: Optional[ModelVendor] = Field(None)
    base_model_id: Optional[str] = Field(None)
    description: Optional[str] = Field(None)
    model_template_id: Optional[str] = Field(None)
    model_creation_parameters: Optional[dict[str, Any]] = Field(None)
    account_id: str = Field(..., description="The ID of the account that owns the given entity.")


class ModelAlias(Entity):
    """
    Entity for all models, including both self-hosted and 3rd party, base- and fine-tuned models.
    """

    name: str
    model_vendor: Optional[ModelVendor] = Field(None)
    base_model_id: Optional[str] = Field(None)
    description: Optional[str] = Field(None)
    model_template_id: Optional[str] = Field(None)
    model_creation_parameters: Optional[dict[str, Any]] = Field(None)
    id: str = Field(..., description="The unique identifier of the entity.")
    created_at: datetime = Field(
        ..., description="The date and time when the entity was created in ISO format."
    )
    account_id: str = Field(..., description="The ID of the account that owns the given entity.")
    created_by_user_id: str = Field(..., description="The user who originally created the entity.")
    status: str = Field(..., description="Status of the model's deployment.")
    request_schema: Dict[str, Any] = Field(
        ..., description="JSON schema for the requests to the model"
    )
    response_schema: Dict[str, Any] = Field(
        ..., description="JSON schema for the response to the model"
    )


class ModelUsageRequest(BaseModel):
    model_id: str
    user_id: str
    started_at: datetime
    finished_at: Optional[datetime] = Field(None)


class ModelUsage(Entity):
    model_id: str
    user_id: str
    started_at: datetime
    finished_at: Optional[datetime] = Field(None)
    id: str = Field(..., description="The unique identifier of the entity.")
    created_at: datetime = Field(
        ..., description="The date and time when the entity was created in ISO format."
    )



import logging
import sys
from argh import ArghParser
from scale_egp.cli.collections import (
    CollectionCRUDCommands,
    EGPClientFactory,
    ModelAliasCommands,
    ModelTemplateCommands,
    UserCommands,
)
from scale_egp.cli.formatter import AVAILABLE_FORMATTERS
from scale_egp.cli.parser import get_parser, dispatch


def add_commands_to_parser(parser: ArghParser, commands: CollectionCRUDCommands):
    parser.add_commands(
        [
            getattr(commands, key)
            for key in dir(commands)
            if not key.startswith("_") and callable(getattr(commands, key))
        ],
        group_name=commands.command_group_name,
    )


def exec_cli():
    client_factory = EGPClientFactory()

    parser = get_parser()
    parser.add_argument("--log-curl-commands", action="store_true", default=False)
    parser.add_argument("-a", "--api-key", type=str, default=None)
    parser.add_argument(
        "-i",
        "--account-id",
        type=str,
        default=None,
        help="Optional EGP account_id to use. The default account_id will be used if not set.",
    )
    parser.add_argument("-e", "--endpoint-url", type=str, default=None)
    parser.add_argument(
        "-f", "--format", type=str, choices=[*AVAILABLE_FORMATTERS.keys()], default="rich"
    )
    add_commands_to_parser(parser, ModelAliasCommands(client_factory))
    add_commands_to_parser(parser, ModelTemplateCommands(client_factory))
    add_commands_to_parser(parser, UserCommands(client_factory))
    args = parser.parse_args()
    try:
        client_factory.set_client_kwargs(
            api_key=args.api_key,
            endpoint_url=args.endpoint_url,
            account_id=args.account_id,
            log_curl_commands=args.log_curl_commands,
        )
    except Exception as e:
        print(f"ERROR: {str(e)}")
        sys.exit(1)
    if args.log_curl_commands:
        logging.basicConfig(level=logging.INFO)
    dispatch(parser, args)



# copied from packages/egp-api-backend/egp_api_backend/server/api/models/egp_models.py
from enum import Enum
from typing import List, Literal, Optional

from pydantic import BaseModel, Field

from .common import BaseModelRequest, BaseModelResponse
from .memory_strategy import LastKMemoryStrategy
from .message import Message

MemoryStrategy = LastKMemoryStrategy


# Messages
class ToolRequest(BaseModel):
    name: str = Field(..., description="Name of the tool that the AI wants the client to use.")
    arguments: str = Field(
        ...,
        description="Arguments to pass to the tool. The format must be a JSON Schema-compliant "
        "object serialized into a string.",
    )


class ActionContext(BaseModel):
    content: Optional[str] = Field(
        default=None,
        description="The final output of the agent when it no longer needs any tools",
    )
    tool_request: Optional[ToolRequest] = Field(
        default=None,
        description="The tool request if the agent needs more information.",
    )


class ToolPropertyValue(BaseModel):
    type: Literal["string", "number", "integer", "boolean", "object", "array", "null",] = Field(
        ...,
        description="The argument's type.\n\n"
        "The type is used to help the Agent generate valid arguments for the "
        "tool.\n\n"
        "For more information about types, see: "
        "https://json-schema.org/understanding-json-schema/reference/type.html#type"
        "-specific-keywords",
    )
    description: str = Field(
        ...,
        description="Description of what the argument is used for.\n\n"
        "This description is used to help the Agent generate sensible arguments for "
        "the tool. It is very important that this description is succinct, clear, "
        "and accurate.",
    )
    default: Optional[str] = Field(description="A default value for the argument if unspecified.")
    examples: Optional[List[str]] = Field(
        description="Example values demonstrating how the argument should look.\n\n"
        "This can be used to help the agent understand what a valid argument should "
        "look like."
    )

    class Config(BaseModel.Config):
        title = "property"


class ToolArguments(BaseModel):
    type: Literal["object"] = Field(
        ..., description='Type of argument. Currently only "object" is supported'
    )
    properties: dict[str, ToolPropertyValue] = Field(
        default_factory=dict,
        description="An object where each key is the name of a keyword argument and each "
        "value is a schema used to validate that property. Each schema must have a "
        "type and description, but can also have a default value and examples.\n\n"
        "For more information on how to define a valid property, visit "
        "https://json-schema.org/understanding-json-schema/reference/object.html",
    )


class Tool(BaseModel):
    name: str = Field(
        ...,
        description="Name of the tool.\n\n"
        "A tool is a function that the _client application_ has at its disposal. The "
        "tool name is the name the client wishes the Agent to use to refer to this "
        "function when it decides if it wants the user to use the tool or not. It"
        "must be unique amongst the set of tools provided in a single API call.",
        pattern=r"^[a-zA-Z0-9_-]{1,64}$",
    )
    description: str = Field(
        ...,
        description="Description of the tool.\n\n"
        "Because some queries are complex and may require multiple "
        "tools to complete, it is important to make these descriptions as "
        "informative as possible. If a tool is not being chosen when it should, "
        "it is common practice to tune the description of the tool to make it more "
        "apparent to the agent when the tool can be used effectively.",
    )
    arguments: ToolArguments = Field(
        ...,
        description="An JSON Schema-compliant schema for the tool arguments. To describe a "
        'function that accepts no parameters, provide the value `{"type": "'
        '"object", "properties": {}}`.\n\n'
        "For more information on how to define a valid JSON Schema object, visit "
        "https://json-schema.org/understanding-json-schema/reference/object.html",
    )


class ExecuteAgentRequest(BaseModelRequest):
    memory_strategy: Optional[MemoryStrategy] = Field(
        default=None,
        description="The memory strategy to use for the agent. A memory strategy "
        "is a way to prevent the underlying LLM's context limit from being exceeded. "
        "Each memory strategy uses a different technique to condense the input message "
        "list into a smaller payload for the underlying LLM.\n\n"
        "We only support the Last K memory strategy right now, but will be adding new "
        "strategies soon.",
    )
    tools: List[Tool] = Field(
        ...,
        description="The list of specs of tools that the agent can use. Each spec must contain "
        "a `name` key set to the name of the tool, a `description` key set to the "
        "description of the tool, and an `arguments` key set to a JSON Schema "
        "compliant object describing the tool arguments.\n\n"
        "The name and description of each tool is used by the agent to decide when to "
        "use certain tools. Because some queries are complex and may require multiple "
        "tools to complete, it is important to make these descriptions as "
        "informative as possible. If a tool is not being chosen when it should, "
        "it is common practice to tune the description of the tool to make it more "
        "apparent to the agent when the tool can be used effectively.\n\n",
    )
    messages: List[Message] = Field(
        ...,
        description="The list of messages in the conversation.\n\n"
        "Expand each message type to see how it works and when to use it. "
        "Most conversations should begin with a single `user` message.",
    )
    instructions: Optional[str] = Field(
        default="You are an AI assistant that helps users with their questions. You "
        "can answer questions directly or acquire information from any of the "
        "attached tools to assist you. Always answer the user's most recent query to the "
        "best of your knowledge.\n\n"
        "When asked about what tools are available, you must list each attached "
        "tool's name and description. When asked about what you can do, mention "
        "that in addition to your normal capabilities, you can also use the attached "
        "tools by listing their names and descriptions. You cannot use any other tools "
        "other than the ones provided to you explicitly.",
        description="The initial instructions to provide to the agent.\n\nUse this to guide the "
        "agent to act in more specific ways. For example, if you have specific "
        "rules you want to restrict the agent to follow you can specify them here. "
        "For example, if I want the agent to always use certain tools before others, "
        "I can write that rule in these instructions.\n\n"
        "Good prompt engineering is crucial to getting performant results from the "
        "agent. If you are having trouble getting the agent to perform well, "
        "try writing more specific instructions here before trying more expensive "
        "techniques such as swapping in other models or finetuning the underlying LLM.",
    )


class AgentAction(str, Enum):
    TOOL_REQUEST = "tool_request"
    CONTENT = "content"


class ExecuteAgentResponse(BaseModelResponse):
    action: AgentAction = Field(
        ...,
        description="The action that the agent performed.\n\n"
        "The context will contain a key for each action that the agent can perform. "
        "However, only the key corresponding to the action that the agent actually "
        "performed will have a populated value. The rest of the values will be `null`.",
    )
    context: ActionContext = Field(
        ...,
        description="Context object containing the output payload. This will contain a key for all "
        "actions that the agent can perform. However, only the key corresponding to "
        "the action that the agent performed have a populated value. The rest of the "
        "values will be `null`.",
    )



import random
import string
from typing import Dict, List, Literal, Optional, Type, TypeVar, Union

from pydantic import BaseModel, Extra, create_model
from pydantic.config import BaseConfig

ParameterValueType = Union[str, int, float, bool]


def get_random_string(length=8):
    # choose from all lowercase letter
    letters = string.ascii_lowercase
    return "".join(random.choice(letters) for i in range(length))


class ParameterSchemaField(BaseModel):
    name: str
    type: Union[Literal["str"], Literal["int"], Literal["float"], Literal["bool"]]
    description: str
    required: bool  # default is optional


class ParameterSchema(BaseModel):
    parameters: List[ParameterSchemaField]


class ParameterBindings(BaseModel):
    bindings: Dict[str, ParameterValueType]

    class Config:
        extra = Extra.forbid


BaseModelT = TypeVar("BaseModelT", bound=BaseModel)


class ParameterSchemaModelConfig(BaseConfig):
    extra = Extra.forbid


def parameter_schema_to_model(
    model_name: str, parameter_schema: ParameterSchema
) -> Type[BaseModelT]:
    """
    Create a Pydantic model from a ParameterSchema
    """
    return create_model(
        model_name,
        __config__=ParameterSchemaModelConfig,
        **{
            field.name: (field.type, ...) if field.required else (field.type, None)
            for field in parameter_schema.parameters
        },
    )


def parse_parameter_bindings(
    parameter_bindings: ParameterBindings,
    parameter_schema: ParameterSchema,
    parameter_schema_name: Optional[str] = None,
) -> BaseModelT:
    model = parameter_schema_to_model(
        parameter_schema_name or get_random_string(), parameter_schema
    )
    return model(**parameter_bindings.bindings)



import enum
import json
from pprint import pformat
import types
from typing import IO, Any, Dict, Iterator, List, Optional

from pydantic import BaseModel, Field

from rich.console import Console
from rich.table import Table
from rich.markdown import Markdown
from rich.text import Text


class Markdownable:
    def to_markdown(self) -> str:
        return self.__repr__()


class FormattingOptions(BaseModel):
    force_list: bool = Field(default=False)
    table_columns: Optional[List[str]] = Field(default=None)


_formatting_options: Optional[FormattingOptions] = None


def set_formatting_options(formatting_options: FormattingOptions):
    global _formatting_options
    _formatting_options = formatting_options


def get_formatting_options() -> FormattingOptions:
    return _formatting_options or FormattingOptions()


class Formatter:
    name = "str"

    def __init__(self, args: Any):
        self.args = args

    def stringify_one(self, value: Any) -> str:
        return str(value)

    def format(self, out_io: IO, lines: Iterator[Any], raw_output: bool = False) -> None:
        # this may raise user exceptions, or SystemExit for wrapped exceptions
        for line in lines:
            # print the line as soon as it is generated to ensure that it is
            # displayed to the user before anything else happens, e.g.
            # raw_input() is called
            out_io.write(self.stringify_one(line))
            if not raw_output:
                # in most cases user wants one message per line
                out_io.write("\n")


class PPrintFormatter(Formatter):
    name = "pprint"

    def stringify_one(self, value: Any) -> str:
        return pformat(value.dict() if isinstance(value, BaseModel) else value)


class JSONFormatter(Formatter):
    name = "json"

    JOINER = ",\n"

    def stringify_one(self, value: Any) -> str:
        return value.json(indent=2) if isinstance(value, BaseModel) else json.dumps(value)

    def format(self, out_io: IO, lines: Iterator[Any], raw_output: bool = False) -> None:
        all_lines = [self.stringify_one(line) for line in lines]
        if not get_formatting_options().force_list and len(all_lines) == 1:
            out_io.write(all_lines[0])
        else:
            out_io.write(f"[{self.JOINER.join(all_lines)}]")


def is_str_enum(cls: types):
    return (
        type(cls) == enum.EnumType
        and cls._member_type_ == str  # pylint:disable=protected-access,unidiomatic-typecheck
    )


class RichFormatter(Formatter):
    name = "rich"
    LIST_JOINER = ", "

    def stringify_one(self, value: Any) -> str:
        if isinstance(value, list):
            return f"[{self.LIST_JOINER.join([self.stringify_one(element) for element in value])}]"
        if is_str_enum(type(value)):
            return value.value
        if isinstance(value, Markdownable):
            return value.to_markdown()
        if isinstance(value, BaseModel):
            return value.json(indent=2)
        if isinstance(value, dict):
            return json.dumps(value, indent=2)
        return super().stringify_one(value)

    def get_column_headers(self, rows: List[Dict[str, Any]]) -> List[str]:
        column_headers = get_formatting_options().table_columns
        if column_headers is None and len(rows) > 0:
            column_headers = sorted(list(rows[0].keys()))
        if column_headers is None:
            column_headers = ["output"]
        return column_headers

    def get_table_title(self) -> str:
        return f"Output of {self.args._functions_stack[0].__self__.command_group_name} {self.args._functions_stack[0].__name__}"

    def get_obj_fields(self, obj: Any) -> Dict[str, Any]:
        return obj if isinstance(obj, dict) else obj.__dict__

    def format_table(self, console: Console, rows: List[Dict[str, Any]], raw_output: bool = False):
        table = Table(title=self.get_table_title())
        column_headers = self.get_column_headers(rows)
        for column_header in column_headers:
            if column_header == "id":
                table.add_column(column_header, style="yellow")
            else:
                table.add_column(column_header)
        for row in rows:
            table.add_row(*[self.stringify_one(row.get(col)) for col in column_headers])
        if raw_output:
            console.out(table)
        else:
            console.print(table)

    def get_rows(self, all_lines: List[Any]) -> List[Dict[str, Any]]:
        if len(all_lines) == 0:
            return []
        sample_line = all_lines[0]
        if isinstance(sample_line, BaseModel):
            return [self.get_obj_fields(line) for line in all_lines]
        if isinstance(sample_line, str):
            return [{"output": line} for line in all_lines]
        return [self.get_obj_fields(line) for line in all_lines]

    def format_object(self, console: Console, obj: Any, raw_output: bool = False):
        output = None
        if isinstance(obj, Markdownable):
            output = Markdown(obj.to_markdown())
        elif isinstance(obj, str):
            output = obj
        else:
            output = Table(title=self.get_table_title())
            output.add_column("field")
            output.add_column("value")
            for field, value in self.get_obj_fields(obj).items():
                output.add_row(field, self.stringify_one(value))
        if raw_output:
            console.out(output)
        else:
            console.print(output)

    def format(self, out_io: IO, lines: Iterator[Any], raw_output: bool = False) -> None:
        console = Console(file=out_io)
        all_lines = list(lines)
        # We want to display results as a table if there's multiple rows or
        # the command forced table view
        # If there are no rows, display this error message
        if len(all_lines) > 0:
            if len(all_lines) > 1 or get_formatting_options().force_list:
                self.format_table(console, self.get_rows(all_lines), raw_output)
            else:
                self.format_object(console, all_lines[0], raw_output)


AVAILABLE_FORMATTERS = {
    formatter_cls.name: formatter_cls for formatter_cls in [*Formatter.__subclasses__(), Formatter]
}



from enum import Enum
from typing import Dict, List, Literal, Optional

from pydantic import Field

try:
    from egp_api_backend.server.api.utils.model_utils import BaseModel
except Exception:
    from pydantic import BaseModel

from .model_enums import ModelEndpointType, ModelVendor


class GPUType(str, Enum):
    # Supported GPU models according to
    # https://github.com/scaleapi/launch-python-client/blob/794089b9ed58330a7ecac02eb5060bcc4ae3d409/launch/client.py#L1470-L1471
    NVIDIA_TESLA_T4 = "nvidia-tesla-t4"
    NVIDIA_AMPERE_A10 = "nvidia-ampere-a10"
    NVIDIA_AMPERE_A100 = "nvidia-ampere-a100"
    NVIDIA_AMPERE_A100e = "nvidia-ampere-a100e"


# The fields in CreateModelBundleConfig(BaseModel) are based on the arguments to
# create_model_bundle_from_runnable_image_v2() in the Launch python client:
# https://github.com/scaleapi/launch-python-client/blob/794089b9ed58330a7ecac02eb5060bcc4ae3d409/launch/client.py#L668
# Differences between this class and the function arguments:
# * Separate fields for docker repository and image name.
# * Omitted fields: tag, healthcheck_route, predict_route, metadata - for these fields EGP user will need to
#   use the defaults provided by Launch.
#
class CreateModelBundleConfig(BaseModel):
    registry: str
    image: str
    tag: str
    command: List[str] = Field(default_factory=list)
    env: Dict[str, str] = Field(default_factory=dict)
    readiness_initial_delay_seconds: int = Field(120)

    @property
    def full_repository_name(self):
        return "/".join([self.registry, self.image])


# The fields in CreateModelEndpointConfig are copied from the arguments of the Launch client's create_model_endpoint()
# https://github.com/scaleapi/launch-python-client/blob/794089b9ed58330a7ecac02eb5060bcc4ae3d409/launch/client.py#L1391
class CreateModelEndpointConfig(BaseModel):
    cpus: int = Field(3)
    memory: str = Field("8Gi")
    storage: str = Field("16Gi")
    gpus: int = Field(0)
    # By default, we create model endpoints with min_workers = 0 so unused model endpoints can be autoscaled down to
    # 0 workers, costing nothing.
    min_workers: int = Field(0)
    max_workers: int = Field(1)
    per_worker: int = Field(10)
    gpu_type: Optional[GPUType] = Field(None)
    endpoint_type: ModelEndpointType = Field(ModelEndpointType.ASYNC)
    high_priority: Optional[bool] = Field(False)


class LaunchVendorConfiguration(BaseModel):
    # this field is required for forward compatibility (other providers will have differend "vendor" fields)
    vendor: Literal[ModelVendor.LLMENGINE] = Field(ModelVendor.LLMENGINE)
    bundle_config: CreateModelBundleConfig
    endpoint_config: Optional[CreateModelEndpointConfig]


# Model vendor configuration only necessary for Launch models currently
ModelVendorConfiguration = LaunchVendorConfiguration



from typing import List, Tuple

from .completions_shared import CompletionBaseRequest, CompletionBaseResponse


class CompletionRequest(CompletionBaseRequest):
    prompts: List[str]


class CompletionResponse(CompletionBaseResponse):
    # List of prompt, completion pairs, eg: [("prompt", ["completion1", "completion2", ...]), ...]
    completions: List[Tuple[str, List[str]]]



from typing import Tuple, Type

from pydantic import BaseModel

# Note: the relative imports here are important since
# these module names must also be valid within model docker containers.
from .agents import ExecuteAgentRequest, ExecuteAgentResponse
from .chat_completions import ChatCompletionRequest, ChatCompletionResponse
from .completions import CompletionRequest, CompletionResponse
from .embedding import EmbeddingRequest, EmbeddingResponse
from .model_enums import ModelType
from .reranking import RerankingRequest, RerankingResponse

MODEL_SCHEMAS: dict[ModelType, Tuple[Type[BaseModel], Type[BaseModel]]] = {
    ModelType.COMPLETION: [CompletionRequest, CompletionResponse],
    ModelType.CHAT_COMPLETION: [ChatCompletionRequest, ChatCompletionResponse],
    ModelType.AGENT: [ExecuteAgentRequest, ExecuteAgentResponse],
    ModelType.RERANKING: [RerankingRequest, RerankingResponse],
    ModelType.EMBEDDING: [EmbeddingRequest, EmbeddingResponse],
}



from typing import Optional, Type

from pydantic import create_model

from .common import BaseModelRequest, BaseModelResponse
from .model_enums import ModelType
from .model_schemas import MODEL_SCHEMAS
from .parameter_schema import ParameterSchema, ParameterSchemaModelConfig, parameter_schema_to_model


def get_full_request_schema(
    model_type: ModelType,
    model_request_parameter_schema: Optional[ParameterSchema] = None,
) -> Type[BaseModelRequest]:
    # dynamically construct Pydantic model which fully describes the model request schema
    # including the model_request_parameters field if it is used by the model.
    request_schema = create_model(
        MODEL_SCHEMAS[model_type][0].__name__,
        __config__=ParameterSchemaModelConfig,
    )
    # copy over all fields except model_request_parameters
    fields = {
        field_name: field_info
        for field_name, field_info in MODEL_SCHEMAS[model_type][0].__fields__.items()
        if field_name != "model_request_parameters"
    }
    if model_request_parameter_schema and len(model_request_parameter_schema.parameters) > 0:
        request_parameter_schema_cls = parameter_schema_to_model(
            "model_request_parameter_schema",
            model_request_parameter_schema,
        )
        fields["model_request_parameters"] = (request_parameter_schema_cls, ...)
    else:
        # permit an empty model_request_parameters to be sent since pydantic will send it automatically
        fields["model_request_parameters"] = BaseModelRequest.__fields__["model_request_parameters"]

    request_schema.__fields__ = fields
    return request_schema


def get_response_schema(model_type: ModelType) -> Type[BaseModelResponse]:
    return MODEL_SCHEMAS[model_type][1]



from typing import List, Tuple

from .common import BaseModelRequest, BaseModelResponse


class EmbeddingRequest(BaseModelRequest):
    texts: List[str]


class EmbeddingResponse(BaseModelResponse):
    # List of text, embedding pairs, eg: [("text to get embedding for", [1.0, 1.05, 2.07, ...]), ...]
    embeddings: List[Tuple[str, List[float]]]



from typing import Literal

from pydantic import BaseModel, Field


# Memory Strategies
class LastKMemoryStrategyParams(BaseModel):
    k: int = Field(
        ...,
        ge=1,
        description="The maximum number of previous messages to remember.",
    )


class LastKMemoryStrategy(BaseModel):
    name: Literal["last_k"] = Field(
        default="last_k",
        const=True,
        description="Name of the memory strategy. Must be `last_k`.\n\n"
        "This strategy truncates the message history to the last `k` messages. It is "
        "the simplest way to prevent the model's context limit from being exceeded. "
        "However, this strategy only allows the model to have short term memory. For "
        "longer term memory, please use one of the other strategies.",
    )
    params: LastKMemoryStrategyParams = Field(
        ..., description="Configuration parameters for the memory strategy."
    )

    class Config(BaseModel.Config):
        title = "Last K Memory Strategy"



import io
import json
from typing import Optional, List, Iterable

import httpx

from scale_egp.sdk.enums import TestCaseSchemaType
from scale_egp.sdk.models import (
    TestCase,
    TestCaseRequest,
    EvaluationDataset,
    EvaluationDatasetVersion,
    EvaluationDatasetVersionRequest,
    TestCaseSchemaValidator,
    TestCaseData,
    EvaluationDatasetRequest,
)
from scale_egp.utils.api_utils import APIEngine


class EvaluationDatasetCollection(APIEngine):
    _sub_path = "v2/evaluation-datasets"

    # START sub collections
    def test_cases(self) -> "TestCaseCollection":
        """
        Returns a TestCaseCollection object for test cases associated with the current Evaluation
        Dataset.
        """
        return TestCaseCollection(self._api_client)

    def _versions(self) -> "EvaluationDatasetVersionCollection":
        """
        Returns a EvaluationDatasetVersionCollection object for versions associated with the current
        Evaluation Dataset.
        """
        return EvaluationDatasetVersionCollection(self._api_client)

    def create(
        self,
        name: str,
        schema_type: TestCaseSchemaType,
        account_id: Optional[str] = None,
    ) -> EvaluationDataset:
        """
        Create a new empty dataset.

        Generally since most users will already have a list of test cases they want to create a
        dataset from, they should use the `create_from_file` method instead.

        Args:
            name: The name of the dataset.
            schema_type: The schema type of the dataset.

        Returns:
            The newly created dataset.
        """
        response = self._post(
            sub_path=self._sub_path,
            request=EvaluationDatasetRequest(
                name=name,
                schema_type=schema_type,
                account_id=account_id or self._api_client.account_id,
            ),
        )
        evaluation_dataset = EvaluationDataset.from_dict(response.json())
        evaluation_dataset = self.update_dataset_version(evaluation_dataset=evaluation_dataset)
        return evaluation_dataset

    def get(
        self,
        id: str,
    ) -> EvaluationDataset:
        """
        Get an existing dataset by ID.

        Args:
            id: The ID of the dataset.

        Returns:
            The dataset.
        """
        latest_dataset_version = self._versions().get_latest(evaluation_dataset_id=id)
        response = self._get(
            sub_path=f"{self._sub_path}/{id}",
        )
        evaluation_dataset = EvaluationDataset.from_dict(response.json())
        if latest_dataset_version:
            evaluation_dataset.version_num = latest_dataset_version.num
        return evaluation_dataset

    def update(
        self,
        id: str,
        name: Optional[str] = None,
        schema_type: Optional[TestCaseSchemaType] = None,
    ) -> EvaluationDataset:
        """
        Update the attributes of an existing dataset.

        **Important**: This method will NOT update the version of the dataset. It will only
        update the attributes of the dataset. If you want to snapshot the current state of the
        dataset under an incremented version number, you should use the `update_dataset_version`
        method instead.

        Args:
            id: The ID of the dataset.
            name: The name of the dataset.
            schema_type: The schema type of the dataset.

        Returns:
            The updated dataset.
        """
        response = self._patch(
            sub_path=f"{self._sub_path}/{id}",
            request=EvaluationDatasetRequest.partial(
                name=name,
                schema_type=schema_type,
            ),
        )
        return EvaluationDataset.from_dict(response.json())

    def delete(
        self,
        id: str,
    ) -> bool:
        """
        Delete an existing dataset.

        Args:
            id: The ID of the dataset.

        Returns:
            True if the dataset was deleted, False otherwise.
        """
        response = self._delete(
            sub_path=f"{self._sub_path}/{id}",
        )
        return response.status_code == httpx.codes.ok

    def list(self) -> List[EvaluationDataset]:
        """
        List all datasets.

        Returns:
            The datasets.
        """
        response = self._get(
            sub_path=self._sub_path,
        )
        return [
            EvaluationDataset.from_dict(evaluation_dataset)
            for evaluation_dataset in response.json()
        ]

    def create_from_file(
        self,
        name: str,
        schema_type: TestCaseSchemaType,
        filepath: str,
    ) -> EvaluationDataset:
        """
        Create a new dataset that is seeded with test cases from a JSONL file.

        The headers of the JSONL file must match the fields of the specified schema type.

        The `schema_types` currently supported and their corresponding fields are:

        **Schema Types:**

        `GENERATION`

        | Field                 | Type             | Default                           |
        | --------------------- | ---------------- | --------------------------------- |
        | `input`               | `str`            | _required_                        |
        | `expected_output`     | `str`            | `None`                            |
        | `expected_extra_info` | `Dict[str, Any]` | `None`                            |

        Args:
            name: The name of the dataset.
            schema_type: The schema type of the dataset.
            filepath: The path to the JSONL file.

        Returns:
            The newly created dataset.
        """
        self._validate_file_type(filepath)
        evaluation_dataset = self.create(
            name=name,
            schema_type=schema_type,
        )
        self._add_test_cases_to_dataset_from_file(
            filepath=filepath,
            evaluation_dataset=evaluation_dataset,
            schema_type=schema_type,
        )
        evaluation_dataset = self.update_dataset_version(evaluation_dataset)
        return evaluation_dataset

    def overwrite_from_file(
        self,
        evaluation_dataset: EvaluationDataset,
        schema_type: TestCaseSchemaType,
        filepath: str,
    ) -> EvaluationDataset:
        """
        Overwrite all test cases in existing dataset from a JSONL file.

        The headers of the JSONL file must match the fields of the specified schema type.

        The `schema_types` currently supported and their corresponding fields are:

        **Schema Types:**

        `GENERATION`

        | Field                 | Type             | Default                           |
        | --------------------- | ---------------- | --------------------------------- |
        | `input`               | `str`            | _required_                        |
        | `expected_output`     | `str`            | `None`                            |
        | `expected_extra_info` | `Dict[str, Any]` | `None`                            |

        Args:
            evaluation_dataset: The dataset to overwrite.
            schema_type: The schema type of the dataset.
            filepath: The path to the JSONL file.

        Returns:
            The updated dataset.
        """
        self._validate_file_type(filepath)
        # Delete all existing test cases
        test_cases_to_delete = self.test_cases().list(evaluation_dataset=evaluation_dataset)
        for test_case in test_cases_to_delete:
            self.test_cases().delete(
                id=test_case.id,
                evaluation_dataset=evaluation_dataset,
            )
        # Update the dataset schema type
        self.update(
            id=evaluation_dataset.id,
            schema_type=schema_type,
        )
        # Add new test cases
        self._add_test_cases_to_dataset_from_file(
            filepath=filepath,
            evaluation_dataset=evaluation_dataset,
            schema_type=schema_type,
        )
        evaluation_dataset = self.update_dataset_version(evaluation_dataset=evaluation_dataset)
        return evaluation_dataset

    def add_test_cases(
        self,
        evaluation_dataset: EvaluationDataset,
        test_cases_data: List[TestCaseData],
        update_dataset_version: bool = True,
    ) -> EvaluationDataset:
        """
        Add new test cases to an existing dataset.

        Unless you want to batch up multiple modifications to a dataset and snapshot them all at
        once, you should leave update_dataset_version=True. See the docs for the
        `update_dataset_version` method for more details.

        The `schema_types` currently supported and their corresponding fields are:

        **Schema Types:**

        `GENERATION`

        | Field                 | Type             | Default                           |
        | --------------------- | ---------------- | --------------------------------- |
        | `input`               | `str`            | _required_                        |
        | `expected_output`     | `str`            | `None`                            |
        | `expected_extra_info` | `Dict[str, Any]` | `None`                            |

        Args:
            evaluation_dataset: The dataset to add test cases to.
            test_cases_data: The test cases to add.
            update_dataset_version: Whether to update the dataset version after adding the test
                cases. Defaults to True.

        Returns:
            The updated dataset.
        """
        self.test_cases().create_batch(
            evaluation_dataset=evaluation_dataset,
            test_cases=[
                TestCaseRequest(
                    evaluation_dataset_id=evaluation_dataset.id,
                    schema_type=evaluation_dataset.schema_type,
                    test_case_data=test_case_data,
                    account_id=self._api_client.account_id,
                )
                for test_case_data in test_cases_data
            ],
        )
        if update_dataset_version:
            evaluation_dataset = self.update_dataset_version(evaluation_dataset=evaluation_dataset)
        return evaluation_dataset

    def add_test_cases_from_file(
        self,
        evaluation_dataset: EvaluationDataset,
        filepath: str,
        update_dataset_version: bool = True,
    ) -> EvaluationDataset:
        """
        Add new test cases to an existing dataset from a JSONL file.

        Unless you want to batch up multiple modifications to a dataset and snapshot them all at
        once, you should leave update_dataset_version=True. See the docs for the
        `update_dataset_version` method for more details.

        The `schema_types` currently supported and their corresponding fields are:

        **Schema Types:**

        `GENERATION`

        | Field                 | Type             | Default                           |
        | --------------------- | ---------------- | --------------------------------- |
        | `input`               | `str`            | _required_                        |
        | `expected_output`     | `str`            | `None`                            |
        | `expected_extra_info` | `Dict[str, Any]` | `None`                            |

        Args:
            evaluation_dataset: The dataset to add test cases to.
            filepath: The path to the JSONL file.
            update_dataset_version: Whether to update the dataset version after adding the test
                cases.

        Returns:
            The updated dataset.
        """
        self._validate_file_type(filepath)
        response = self._get(
            sub_path=f"{self._sub_path}/{evaluation_dataset.id}",
        )
        evaluation_dataset = EvaluationDataset.from_dict(response.json())
        self._add_test_cases_to_dataset_from_file(
            filepath=filepath,
            evaluation_dataset=evaluation_dataset,
            schema_type=evaluation_dataset.schema_type,
        )
        if update_dataset_version:
            evaluation_dataset = self.update_dataset_version(evaluation_dataset=evaluation_dataset)
        return evaluation_dataset

    def delete_test_cases(
        self,
        evaluation_dataset: EvaluationDataset,
        test_case_ids: List[str],
        update_dataset_version: bool = True,
    ) -> EvaluationDataset:
        """
        Delete test cases from an existing dataset.

        Unless you want to batch up multiple modifications to a dataset and snapshot them all at
        once, you should leave update_dataset_version=True. See the docs for the
        `update_dataset_version` method for more details.

        Args:
            evaluation_dataset: The dataset to delete test cases from.
            test_case_ids: The IDs of the test cases to delete.
            update_dataset_version: Whether to update the dataset version after deleting the test
                cases.

        Returns:
            The updated dataset.
        """
        for test_case_id in test_case_ids:
            self.test_cases().delete(
                id=test_case_id,
                evaluation_dataset=evaluation_dataset,
            )
        if update_dataset_version:
            evaluation_dataset = self.update_dataset_version(evaluation_dataset=evaluation_dataset)
        return evaluation_dataset

    def modify_test_cases(
        self,
        evaluation_dataset: EvaluationDataset,
        modified_test_cases: List[TestCase],
        update_dataset_version: bool = True,
    ) -> EvaluationDataset:
        """
        Modify test cases in an existing dataset.

        Unless you want to batch up multiple modifications to a dataset and snapshot them all at
        once, you should leave update_dataset_version=True. See the docs for the
        `update_dataset_version` method for more details.

        Args:
            evaluation_dataset: The dataset to modify test cases in.
            modified_test_cases: The modified test cases.
            update_dataset_version: Whether to update the dataset version after modifying the test
                cases.

        Returns:
            The updated dataset.
        """
        for test_case in modified_test_cases:
            self.test_cases().update(
                test_case_id=test_case.id,
                evaluation_dataset=evaluation_dataset,
                schema_type=test_case.schema_type,
                test_case_data=test_case.test_case_data,
            )
        if update_dataset_version:
            evaluation_dataset = self.update_dataset_version(evaluation_dataset=evaluation_dataset)
        return evaluation_dataset

    def update_dataset_version(self, evaluation_dataset: EvaluationDataset) -> EvaluationDataset:
        """
        Update the version of an existing dataset.

        This method will snapshot the current state of the dataset under an incremented version
        number.

        **Warning**: By default, the `add_test_cases`, `delete_test_cases`, and `modify_test_cases`
        methods will automatically update the dataset version for you. However,
        if you want to batch up multiple modifications to a dataset and snapshot them all at once,
        you can set `update_dataset_version=False` on those methods and then call this method
        manually afterward.

        Args:
            evaluation_dataset: The dataset to update the version of.

        Returns:
            The updated dataset.
        """
        evaluation_dataset_version = self._versions().create(evaluation_dataset=evaluation_dataset)
        evaluation_dataset.version_num = evaluation_dataset_version.num
        return evaluation_dataset

    def _add_test_cases_to_dataset_from_file(
        self, filepath: str, evaluation_dataset: EvaluationDataset, schema_type: TestCaseSchemaType
    ):
        self._validate_file_type(filepath)
        test_cases = []
        with io.open(filepath, "r", encoding="utf-8") as f:
            for line in f:
                line = line.strip()
                if not line:
                    continue
                json_data = json.loads(line)
                test_cases.append(
                    TestCaseRequest(
                        evaluation_dataset_id=evaluation_dataset.id,
                        schema_type=schema_type,
                        test_case_data=TestCaseSchemaValidator.dict_to_model(
                            schema_type=schema_type,
                            data=json_data.items(),
                        ),
                        account_id=self._api_client.account_id,
                    )
                )
        self.test_cases().create_batch(
            evaluation_dataset=evaluation_dataset,
            test_cases=test_cases,
        )

    @staticmethod
    def _validate_file_type(filepath: str):
        if not filepath.endswith(".jsonl"):
            raise ValueError(
                "The filepath provided must be a JSONL file. "
                "See https://jsonlines.org/ for more details."
            )


class EvaluationDatasetVersionCollection(APIEngine):
    _sub_path = "v2/evaluation-datasets/{evaluation_dataset_id}/evaluation-dataset-versions"

    def create(
        self,
        evaluation_dataset: EvaluationDataset,
    ) -> EvaluationDatasetVersion:
        """
        Create a new dataset version.

        Args:
            evaluation_dataset: The dataset to create a version for.

        Returns:
            The newly created dataset version.
        """
        response = self._post(
            sub_path=self._sub_path.format(evaluation_dataset_id=evaluation_dataset.id),
            request=EvaluationDatasetVersionRequest(
                evaluation_dataset_id=evaluation_dataset.id,
                account_id=evaluation_dataset.account_id or self._api_client.account_id,
            ),
        )
        return EvaluationDatasetVersion.from_dict(response.json())

    def get(
        self,
        id: str,
        evaluation_dataset: EvaluationDataset,
    ) -> EvaluationDatasetVersion:
        """
        Get an existing dataset version by ID.

        Args:
            id: The ID of the dataset version.
            evaluation_dataset: The dataset to get the version from.

        Returns:
            The dataset version.
        """
        response = self._get(
            sub_path=f"{self._sub_path.format(evaluation_dataset_id=evaluation_dataset.id)}/{id}",
        )
        return EvaluationDatasetVersion.from_dict(response.json())

    def list(
        self,
        evaluation_dataset: EvaluationDataset,
    ) -> List[EvaluationDatasetVersion]:
        """
        List all versions of a dataset.

        Args:
            evaluation_dataset: The dataset to list versions from.

        Returns:
            The dataset versions.
        """
        response = self._get(
            sub_path=self._sub_path.format(evaluation_dataset_id=evaluation_dataset.id),
        )
        return [
            EvaluationDatasetVersion.from_dict(evaluation_dataset_version)
            for evaluation_dataset_version in response.json()
        ]

    def get_latest(
        self,
        evaluation_dataset_id: str,
    ) -> Optional[EvaluationDatasetVersion]:
        """
        Get the latest version of a dataset.

        Args:
            evaluation_dataset_id: The ID of the dataset.

        Returns:
            The latest dataset version, or None if no versions exist.
        """
        response = self._get(
            sub_path=self._sub_path.format(evaluation_dataset_id=evaluation_dataset_id),
        )
        existing_dataset_versions = [
            EvaluationDatasetVersion.from_dict(evaluation_dataset_version)
            for evaluation_dataset_version in response.json()
        ]
        if existing_dataset_versions:
            return sorted(existing_dataset_versions, key=lambda x: x.num)[-1]
        return None


class TestCaseCollection(APIEngine):
    _sub_path = "v2/evaluation-datasets/{evaluation_dataset_id}/test-cases"

    def create(
        self,
        evaluation_dataset: EvaluationDataset,
        schema_type: TestCaseSchemaType,
        test_case_data: TestCaseData,
    ) -> TestCase:
        """
        Create a new test case.

        Args:
            evaluation_dataset: The dataset to create the test case in.
            schema_type: The schema type of the test case.
            test_case_data: The test case data.

        Returns:
            The newly created test case.
        """
        response = self._post(
            sub_path=self._sub_path.format(evaluation_dataset_id=evaluation_dataset.id),
            request=TestCaseRequest(
                evaluation_dataset_id=evaluation_dataset.id,
                schema_type=schema_type,
                test_case_data=test_case_data,
                account_id=self._api_client.account_id,
            ),
        )
        return TestCase.from_dict(response.json())

    def create_batch(
        self,
        evaluation_dataset: EvaluationDataset,
        test_cases: List[TestCaseRequest],
    ) -> List[TestCase]:
        """
        Create multiple new test cases.

        Args:
            evaluation_dataset: The dataset to create the test cases in.
            test_cases: The test cases to create.

        Returns:
            The newly created test cases.
        """
        response = self._post_batch(
            sub_path=f"{self._sub_path.format(evaluation_dataset_id=evaluation_dataset.id)}/batch",
            request_batch=test_cases,
        )
        return [TestCase.from_dict(test_case) for test_case in response.json()]

    def get(
        self,
        id: str,
        evaluation_dataset: EvaluationDataset,
    ) -> TestCase:
        """
        Get an existing test case by ID.

        Args:
            id: The ID of the test case.
            evaluation_dataset: The dataset to get the test case from.

        Returns:
            The test case.
        """
        response = self._get(
            sub_path=f"{self._sub_path.format(evaluation_dataset_id=evaluation_dataset.id)}/{id}",
        )
        return TestCase.from_dict(response.json())

    def list(self, evaluation_dataset: EvaluationDataset) -> List[TestCase]:
        """
        List all test cases in a dataset.

        Args:
            evaluation_dataset: The dataset to list test cases from.

        Returns:
            The test cases.
        """
        response = self._get(
            sub_path=self._sub_path.format(evaluation_dataset_id=evaluation_dataset.id),
        )
        return [TestCase.from_dict(test_case) for test_case in response.json()]

    def update(
        self,
        test_case_id: str,
        evaluation_dataset: EvaluationDataset,
        schema_type: Optional[TestCaseSchemaType] = None,
        test_case_data: Optional[TestCaseData] = None,
    ) -> TestCase:
        """
        Update an existing test case.

        Args:
            test_case_id: The ID of the test case.
            evaluation_dataset: The dataset to update the test case in.
            schema_type: The schema type of the test case.
            test_case_data: The test case data.

        Returns:
            The updated test case.
        """
        response = self._patch(
            sub_path=f"{self._sub_path.format(evaluation_dataset_id=evaluation_dataset.id)}/"
            f"{test_case_id}",
            request=TestCaseRequest.partial(
                schema_type=schema_type,
                test_case_data=test_case_data,
            ),
        )
        return TestCase.from_dict(response.json())

    def iter(self, evaluation_dataset: EvaluationDataset) -> Iterable[TestCase]:
        """
        Iterate over all test cases in a dataset.

        Args:
            evaluation_dataset: The dataset to iterate over test cases from.

        Returns:
            The test cases.
        """
        response = self._get(
            sub_path=self._sub_path.format(evaluation_dataset_id=evaluation_dataset.id),
        )
        for test_case in response.json():
            yield TestCase.from_dict(test_case)

    def delete(
        self,
        id: str,
        evaluation_dataset: EvaluationDataset,
    ) -> bool:
        """
        Delete an existing test case.

        Args:
            id: The ID of the test case.
            evaluation_dataset: The dataset to delete the test case from.

        Returns:
            True if the test case was deleted successfully, False otherwise.
        """
        response = self._delete(
            sub_path=f"{self._sub_path.format(evaluation_dataset_id=evaluation_dataset.id)}/{id}",
        )
        return response.status_code == httpx.codes.ok



import json
from typing import Any, Dict, Generator, Generic, Iterator, Optional, Type, TypeVar
from scale_egp.cli.formatter import (
    FormattingOptions,
    get_formatting_options,
    set_formatting_options,
)
from scale_egp.cli.model_description import ModelDescription
from scale_egp.sdk.client import EGPClient
from scale_egp.sdk.collections.models_api.model_templates import ModelTemplateCollection
from scale_egp.sdk.collections.models_api.models import ModelCollection
from scale_egp.sdk.models.user_info import UserInfoResponse, get_user_info
from scale_egp.sdk.models.model_api_models import (
    ModelAlias,
    ModelAliasRequest,
    ModelTemplate,
    ModelTemplateRequest,
)
from scale_egp.sdk.models.model_schemas.model_schemas import MODEL_SCHEMAS
from scale_egp.utils.api_utils import APIEngine
from scale_egp.utils.model_utils import BaseModel
from argh import CommandError, arg
import fastjsonschema


EntityT = TypeVar("EntityT", bound=BaseModel)
RequestT = TypeVar("RequestT", bound=BaseModel)


def read_json_file(filename: str) -> Any:
    with open(filename, "r", encoding="utf-8") as f:
        return json.loads(f.read())


class EGPClientFactory:
    def __init__(
        self,
    ):
        self.client: Optional[EGPClient] = None
        self._client_kwargs = None

    def set_client_kwargs(self, **kwargs):
        self._client_kwargs = kwargs

    def get_client(self) -> EGPClient:
        if self.client is None:
            self.client = EGPClient(**self._client_kwargs)
        return self.client


class CollectionCRUDCommands(Generic[EntityT, RequestT]):
    command_group_name = "CRUD"

    def __init__(
        self,
        client_factory: EGPClientFactory,
        entity_type: Type[EntityT],
        request_type: Type[RequestT],
        collection_type: Type[APIEngine],
    ):
        self._client_factory = client_factory
        self._entity_type = entity_type
        self._request_type = request_type
        self._collection_type = collection_type

    def _get_collection_instance(self) -> APIEngine:
        return self._collection_type(self._client_factory.get_client())

    def _transform_entity_json(self, entity_dict: Dict[str, Any]) -> Dict[str, Any]:
        return entity_dict

    def _create(self, request_dict: Any) -> EntityT:
        assert isinstance(request_dict, dict)
        # add client account id if not set in file
        request_dict["account_id"] = request_dict.get(
            "account_id", self._client_factory.get_client().account_id
        )
        request_obj = self._request_type(**request_dict)
        collection = self._get_collection_instance()
        response = collection._post(getattr(collection, "_sub_path"), request_obj)
        assert response.status_code == 200
        response_dict = response.json()
        assert isinstance(response_dict, dict)
        return self._entity_type(**response_dict)

    @arg("filename", help="file to load")
    def create(self, filename: str) -> EntityT:
        request_dict = read_json_file(filename)
        return self._create(request_dict)

    def get(self, id: str) -> EntityT:
        collection = self._get_collection_instance()
        sub_path = f"{collection._sub_path}/{id}"
        response = collection._get(sub_path)
        assert response.status_code == 200
        response_dict = response.json()
        assert isinstance(response_dict, dict)
        return self._entity_type(**response_dict)

    def list(self) -> Generator[EntityT, None, None]:
        formatting_options = get_formatting_options()
        formatting_options.force_list = True
        set_formatting_options(formatting_options)
        collection = self._get_collection_instance()
        response = collection._get(collection._sub_path)
        assert response.status_code == 200
        response_list = response.json()
        assert isinstance(response_list, list)
        # TODO: pagination
        for entity_dict in response_list:
            yield self._entity_type(**entity_dict)


class ModelAliasCommands(CollectionCRUDCommands[ModelAlias, ModelAliasRequest]):
    command_group_name = "model"

    def __init__(
        self,
        client_factory: EGPClientFactory,
    ):
        super().__init__(client_factory, ModelAlias, ModelAliasRequest, ModelCollection)

    def list(self) -> Generator[EntityT, None, None]:
        set_formatting_options(
            FormattingOptions(
                table_columns=["id", "name", "model_template_id", "created_at", "description"]
            )
        )
        return super().list()

    def describe(self, model_id: str) -> ModelDescription:
        model_alias = self.get(model_id)
        model_template = ModelTemplateCommands(self._client_factory).get(
            model_alias.model_template_id
        )
        return ModelDescription(model_instance=model_alias, model_template=model_template)

    @arg("filename", help="file to load")
    @arg(
        "--model-template-id", help="id of the model template to use if not specified in JSON file"
    )
    def create(self, filename: str, model_template_id: Optional[str] = None) -> EntityT:
        request_dict = read_json_file(filename)
        assert isinstance(request_dict, dict)
        effective_model_template_id = request_dict.get("model_template_id", model_template_id)
        if effective_model_template_id is None:
            raise CommandError(
                "No model template id specified in JSON file or --model-template-id option. Please provide the model template id!"
            )
        request_dict["model_template_id"] = effective_model_template_id
        return self._create(request_dict)

    @arg("filename", help="Model request")
    def validate_request(self, model_id: str, filename: str) -> Optional[str]:
        model_alias = self.get(model_id)
        execute_request_dict = read_json_file(filename)
        validator = fastjsonschema.compile(model_alias.request_schema)
        try:
            validator(execute_request_dict)
        except fastjsonschema.JsonSchemaException as e:
            return f"Data failed validation: {e}"
        return None

    @arg("filename", help="Model request")
    def execute(self, model_id: str, filename: str) -> EntityT:
        model_alias = self.get(model_id)
        model_template = ModelTemplateCommands(self._client_factory).get(
            model_alias.model_template_id
        )
        model_type = model_template.model_type
        model_request_cls, model_response_cls = MODEL_SCHEMAS[model_type]
        request_dict = read_json_file(filename)
        assert isinstance(request_dict, dict)
        request_obj = model_request_cls(**request_dict)
        collection = self._get_collection_instance()
        sub_path = f"{collection._sub_path}/{model_id}/execute"
        response = collection._post(
            sub_path=sub_path,
            request=request_obj,
            timeout=10 * 60,  # 10 minutes
        )
        assert response.status_code == 200
        return model_response_cls(**response.json())


class ModelTemplateCommands(CollectionCRUDCommands[ModelTemplate, ModelTemplateRequest]):
    command_group_name = "model_template"

    def __init__(
        self,
        client_factory: EGPClientFactory,
    ):
        super().__init__(
            client_factory, ModelTemplate, ModelTemplateRequest, ModelTemplateCollection
        )

    def _transform_entity_json(self, entity_dict: Dict[str, Any]) -> Dict[str, Any]:
        if entity_dict.get("vendor_configuration") is not None:
            entity_dict["model_vendor"] = "LLMENGINE"
        return entity_dict

    def list(self) -> Generator[EntityT, None, None]:
        set_formatting_options(
            FormattingOptions(table_columns=["id", "name", "model_type", "created_at"])
        )
        return super().list()

    def show_model_schemas(self):
        return [
            {
                "model_type": model_type.value,
                "request_schema": schemas[0].schema(),
                "response_schema": schemas[1].schema(),
            }
            for (model_type, schemas) in MODEL_SCHEMAS.items()
        ]


class UserCommands:
    command_group_name = "user"

    def __init__(
        self,
        client_factory: EGPClientFactory,
    ):
        self.client_factory = client_factory

    def whoami(self) -> UserInfoResponse:
        client = self.client_factory.get_client()
        return get_user_info(client.httpx_client, client.endpoint_url, client.log_curl_commands)



import functools
import json
import logging

from typing import TYPE_CHECKING, Any, Callable, Dict, Iterable, List, Optional
from urllib.parse import urlencode, urljoin

from httpx import Client, Response, Timeout
from scale_egp.exceptions import exception_from_response
from scale_egp.utils.model_utils import BaseModel

if TYPE_CHECKING:
    from scale_egp.sdk.client import EGPClient

DEFAULT_TIMEOUT = 60

ten_minutes = 60 * 10
long_timeout = Timeout(connect=ten_minutes, read=ten_minutes, write=ten_minutes, pool=ten_minutes)
CURL_DEFAULT_HEADERS = {"Content-Type": "application/json"}

curl_logger = logging.getLogger("curl_command_log")
curl_logger.setLevel(logging.INFO)


def handle_api_exceptions(func: Callable) -> Callable:
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        response = func(*args, **kwargs)
        if response.status_code != 200:
            raise exception_from_response(response)
        return response

    return wrapper


def log_curl_command(
    client: Client,
    method: str,
    url: str,
    additional_headers: Optional[Dict[str, str]] = None,
    request: Optional[Dict[str, Any]] = None,
):
    cmd_parts = (
        ["curl", f"-X {method}", url]
        + [
            f"-H '{name}: {value}'"
            for (name, value) in {
                **CURL_DEFAULT_HEADERS,
                **(additional_headers or {}),
                **client.headers,
            }.items()
        ]
        + ([f"-d '{json.dumps(request)}'"] if request is not None else [])
    )
    curl_logger.info(" ".join(cmd_parts))


class APIEngine:
    def __init__(self, api_client: "EGPClient"):
        self._api_client = api_client

    def _log_curl_command(
        self,
        method: str,
        url: str,
        additional_headers: Optional[Dict[str, str]] = None,
        request: Optional[Dict[str, Any]] = None,
    ):
        if self._api_client.log_curl_commands:
            log_curl_command(
                self._api_client.httpx_client, method, url, additional_headers, request
            )

    def _post(
        self,
        sub_path: str,
        request: Optional[BaseModel] = None,
        timeout: Optional[int] = None,
    ) -> Response:
        response = self._raw_post(
            sub_path=sub_path,
            request_json=request.dict() if request is not None else None,
            timeout=timeout if timeout is not None else DEFAULT_TIMEOUT,
        )
        return response

    def _post_stream(
        self,
        sub_path: str,
        request: Optional[BaseModel] = None,
        timeout: Optional[int] = None,
    ) -> Iterable[Dict[str, Any]]:
        response = self._raw_stream(
            sub_path=sub_path,
            request_json=request.dict() if request is not None else None,
            timeout=timeout if timeout is not None else DEFAULT_TIMEOUT,
        )
        with response as lines:
            for line in lines.iter_lines():
                if line.startswith("data: "):
                    event_json_str = line[len("data: ") :]
                    try:
                        yield json.loads(event_json_str)
                    except json.JSONDecodeError:
                        raise ValueError(f"Invalid JSON payload: {event_json_str}")

    def _post_batch(
        self,
        sub_path: str,
        request_batch: Optional[List[BaseModel]] = None,
        timeout: Optional[int] = None,
    ) -> Response:
        response = self._raw_post(
            sub_path=sub_path,
            request_json=[request.dict() for request in request_batch],
            timeout=timeout if timeout is not None else DEFAULT_TIMEOUT,
        )
        return response

    def _get(
        self,
        sub_path: str,
        query_params: Optional[Dict[str, str]] = None,
        timeout: Optional[int] = None,
    ) -> Response:
        response = self._raw_get(
            sub_path=sub_path,
            query_params=query_params,
            timeout=timeout if timeout is not None else DEFAULT_TIMEOUT,
        )
        return response

    def _patch(
        self,
        sub_path: str,
        request: Optional[BaseModel] = None,
        timeout: Optional[int] = None,
    ) -> Response:
        response = self._raw_patch(
            sub_path=sub_path,
            request_json=request.dict() if request is not None else None,
            timeout=timeout if timeout is not None else DEFAULT_TIMEOUT,
        )
        return response

    def _delete(
        self,
        sub_path: str,
        timeout: Optional[int] = None,
    ) -> Response:
        response = self._raw_delete(
            sub_path=sub_path,
            timeout=timeout if timeout is not None else DEFAULT_TIMEOUT,
        )
        return response

    @handle_api_exceptions
    def _raw_post(
        self,
        sub_path: str,
        request_json: Optional[Dict[str, Any]] = None,
        timeout: Optional[int] = None,
        additional_headers: Optional[Dict[str, str]] = None,
    ) -> Response:
        url = urljoin(self._api_client.endpoint_url, sub_path)
        self._log_curl_command("POST", url, additional_headers, request_json)
        response = self._api_client.httpx_client.post(
            url,
            json=request_json,
            **self._universal_request_kwargs(
                timeout=timeout,
                additional_headers=additional_headers,
            ),
        )
        return response

    def _raw_stream(
        self,
        sub_path: str,
        request_json: Optional[Dict[str, Any]] = None,
        timeout: Optional[int] = None,
        additional_headers: Optional[Dict[str, str]] = None,
        method: str = "POST",
    ):
        url = urljoin(self._api_client.endpoint_url, sub_path)
        self._log_curl_command("POST", url, additional_headers, request_json)
        return self._api_client.httpx_client.stream(
            method=method,
            url=url,
            json=request_json,
            timeout=timeout if timeout is not None else DEFAULT_TIMEOUT,
            headers={
                **(additional_headers if additional_headers is not None else {}),
            },
        )

    @handle_api_exceptions
    def _raw_get(
        self,
        sub_path: str,
        timeout: Optional[int] = None,
        query_params: Optional[Dict[str, str]] = None,
        additional_headers: Optional[Dict[str, str]] = None,
    ) -> Response:
        url = urljoin(self._api_client.endpoint_url, sub_path)
        self._log_curl_command(
            "GET",
            url if query_params is None else f"{url}?{urlencode(query_params)}",
            additional_headers,
        )
        response = self._api_client.httpx_client.get(
            url,
            params=query_params,
            **self._universal_request_kwargs(
                timeout=timeout,
                additional_headers=additional_headers,
            ),
        )
        return response

    @handle_api_exceptions
    def _raw_patch(
        self,
        sub_path: str,
        request_json: Optional[Dict[str, Any]] = None,
        timeout: Optional[int] = None,
        additional_headers: Optional[Dict[str, str]] = None,
    ) -> Response:
        url = urljoin(self._api_client.endpoint_url, sub_path)
        self._log_curl_command("PATCH", url, additional_headers, request_json)
        response = self._api_client.httpx_client.patch(
            url,
            json=request_json,
            **self._universal_request_kwargs(
                timeout=timeout,
                additional_headers=additional_headers,
            ),
        )
        return response

    @handle_api_exceptions
    def _raw_delete(
        self,
        sub_path: str,
        timeout: Optional[int] = None,
        additional_headers: Optional[Dict[str, str]] = None,
    ) -> Response:
        url = urljoin(self._api_client.endpoint_url, sub_path)
        self._log_curl_command("PATCH", url, additional_headers)
        response = self._api_client.httpx_client.delete(
            url=url,
            **self._universal_request_kwargs(
                timeout=timeout,
                additional_headers=additional_headers,
            ),
        )
        return response

    def _universal_request_kwargs(
        self,
        timeout: Optional[int] = None,
        additional_headers: Optional[Dict[str, str]] = None,
    ) -> Dict[str, Any]:
        return dict(
            timeout=timeout if timeout is not None else DEFAULT_TIMEOUT,
            headers={
                **(additional_headers if additional_headers is not None else {}),
            },
        )



import json
from typing import Any, Callable, Dict, Literal, Optional, Type, Union

from pydantic import BaseModel
from scale_egp.cli.formatter import Markdownable
from scale_egp.sdk.models.model_api_models import ModelAlias, ModelTemplate
from scale_egp.sdk.models.model_schemas.model_enums import ModelVendor
from scale_egp.sdk.models.model_schemas.request_schema import get_full_request_schema
from scale_egp.sdk.models.model_schemas.model_schemas import MODEL_SCHEMAS
from scale_egp.sdk.models.model_schemas.parameter_schema import parameter_schema_to_model


class ModelDescription(BaseModel, Markdownable):
    model_instance: ModelAlias
    model_template: ModelTemplate

    def _get_request_schema(self) -> Dict[str, Any]:
        return get_full_request_schema(
            self.model_template.model_type,
            self.model_template.model_request_parameters_schema,
        ).schema()

    def _get_description_dict(self) -> Dict[str, Any]:
        return {
            "model_instance": json.loads(self.model_instance.json()),
            "model_template": json.loads(self.model_template.json()),
        }

    def to_markdown(self) -> str:
        return (
            f"# {self.model_instance.name} (id: {self.model_instance.id})\n"
            f"\n"
            f"*type*: {self.model_template.model_type.value}\n"
            f"*status*: {self.model_instance.status}\n"
            f"*vendor*: {(self.model_instance.model_vendor or ModelVendor.LLMENGINE).value}\n"
            f"\n"
            f"{self.model_instance.description or ''}\n"
            f"## Model request schema\n"
            f"```\n"
            f"{json.dumps(self.model_instance.request_schema, indent=2)}\n"
            f"```\n"
            f"## Model response schema\n"
            f"```\n"
            f"{json.dumps(self.model_instance.response_schema, indent=2)}\n"
            f"```\n"
        )

    def json(self, **dumps_kwargs: Any) -> str:
        return json.dumps(self._get_description_dict(), **dumps_kwargs)



from enum import Enum


class ModelState(str, Enum):
    ENABLED = "ENABLED"
    PENDING = "PENDING"
    DISABLED = "DISABLED"


class ModelVendor(str, Enum):
    OPENAI = "OPENAI"
    COHERE = "COHERE"
    ANTHROPIC = "ANTHROPIC"
    LLMENGINE = "LLMENGINE"
    OTHER = "OTHER"


class ModelEndpointType(str, Enum):
    SYNC = "SYNC"
    ASYNC = "ASYNC"
    STREAMING = "STREAMING"
    BATCH = "BATCH"


class ModelType(str, Enum):
    COMPLETION = "COMPLETION"
    CHAT_COMPLETION = "CHAT_COMPLETION"
    AGENT = "AGENT"
    EMBEDDING = "EMBEDDING"
    RERANKING = "RERANKING"



from typing import List, Optional

from pydantic import Field

from .common import BaseModelRequest, BaseModelResponse


class CompletionBaseRequest(BaseModelRequest):
    temperature: float = Field(
        default=0.2,
        ge=0.0,
        le=1.0,
        description="What sampling temperature to use, between [0, 1]. Higher values like 0.8 "
        "will make the output more random, while lower values like 0.2 will make it "
        "more focused and deterministic. Setting temperature=0.0 will enable fully "
        "deterministic "
        "(greedy) sampling.",
    )
    stop_sequences: Optional[List[str]] = Field(
        default=None,
        max_items=4,
        description="List of up to 4 sequences where the API will stop generating further tokens. "
        "The returned text will not contain the stop sequence.",
    )
    max_tokens: Optional[int] = Field(
        default=None,
        description="The maximum number of tokens to generate in the completion. The token count "
        "of your prompt plus max_tokens cannot exceed the model's context length. If not, "
        "specified, max_tokens will be determined based on the model used: \n"
        "| Model API family | Model API default | EGP applied default |\n"
        "| --- | --- | --- |\n"
        "| OpenAI Completions | [`16`](https://platform.openai.com/docs/api-reference/completions/create#max_tokens) | `context window - prompt size` |\n"
        "| OpenAI Chat Completions | [`context window - prompt size`](https://platform.openai.com/docs/api-reference/chat/create#max_tokens) | `context window - prompt size` |\n"
        "| LLM Engine | [`max_new_tokens`](https://github.com/scaleapi/launch-python-client/blob/207adced1c88c1c2907266fa9dd1f1ff3ec0ea5b/launch/client.py#L2910) parameter is required | `100` |\n"
        "| Athropic Claude 2 | [`max_tokens_to_sample`](https://docs.anthropic.com/claude/reference/complete_post) parameter is required | `10000` |\n",
    )


class CompletionBaseResponse(BaseModelResponse):
    finish_reason: Optional[str]



from enum import Enum


class TestCaseSchemaType(str, Enum):
    """
    An enum representing the different test case schema types.

    Attributes:
        GENERATION: If a TestCase specifies this schema type, it must have the following fields:

            - `input` [Required] The input to the generation model.
            - `expected_output` [Optional] The expected output of the generation model.
            - `expected_extra_info` [Optional] The expected extra info of the generation model.

            If a TestCaseResult specifies this schema type, it must have the following fields:

            - `generation_output` [Required] The output of the generation model.
            - `generation_extra_info` [Required] The extra info of the generation model.
    """

    GENERATION: str = "GENERATION"


class EvaluationStatus(str, Enum):
    """
    An enum representing the differ possible statuses of an Evaluation.

    Attributes:
        PENDING: Denotes that an evaluation is pending.
        COMPLETED: Denotes that an evaluation is completed.
        FAILED: Denotes that an evaluation has failed.
    """

    PENDING: str = "PENDING"
    COMPLETED: str = "COMPLETED"
    FAILED: str = "FAILED"


class EvaluationType(str, Enum):
    """
    An enum representing the different types of evaluations.

    Currently only studio evaluations are supported.

    Attributes:
        STUDIO: Denotes that an evaluation is a studio evaluation.
    """

    STUDIO: str = "studio"


class QuestionType(str, Enum):
    """
    An enum representing the different types of questions.

    This is only used in a StudioEvaluationConfig to specify the type of a question.

    Attributes:
        CATEGORICAL: Denotes that a question is a categorical question.
        DROPDOWN: Denotes that a question is a dropdown question.
        FREE_TEXT: Denotes that a question is a free text question.
    """

    CATEGORICAL: str = "categorical"
    DROPDOWN: str = "dropdown"
    FREE_TEXT: str = "free_text"


class ExtraInfoSchema(str, Enum):
    """
    An enum representing the different types of extra info schemas.

    Denotes the type of the "info" field in the ExtraInfo model.

    Attributes:
        STRING: Denotes that the "info" field is a string.
    """

    STRING: str = "STRING"


class EmbeddingModelName(str, Enum):
    """
    An enum representing the different types of embedding models supported.

    Attributes:
        SENTENCE_TRANSFORMERS_ALL_MINILM_L12_V2:
            Denotes that the model is a sentence transformer model.
        SENTENCE_TRANSFORMERS_MULTI_QA_DISTILBERT_COS_V1:
            Denotes that the model is a sentence transformer model.
        SENTENCE_TRANSFORMERS_PARAPHRASE_MULTILINGUAL_MPBET_BASE_V2:
            Denotes that the model is a sentence transformer model.
        OPENAI_TEXT_EMBEDDING_ADA_002:
            Denotes that the model is an openai text embedding model.
    """

    SENTENCE_TRANSFORMERS_ALL_MINILM_L12_V2 = "sentence-transformers/all-MiniLM-L12-v2"
    SENTENCE_TRANSFORMERS_MULTI_QA_DISTILBERT_COS_V1 = (
        "sentence-transformers/multi-qa-distilbert-cos-v1"
    )
    SENTENCE_TRANSFORMERS_PARAPHRASE_MULTILINGUAL_MPBET_BASE_V2 = (
        "sentence-transformers/paraphrase-multilingual-mpnet-base-v2"
    )
    OPENAI_TEXT_EMBEDDING_ADA_002 = "openai/text-embedding-ada-002"
    SENTENCE_TRANSFORMERS_MULTI_QA_FINETUNE = "finetuned/multi-qa-mpnet-base-dot-v1"


class DataSource(str, Enum):
    """
    An enum representing the different types of data sources supported.

    Attributes:
        S3: Denotes that the data source is S3.
    """

    S3: str = "S3"
    SHAREPOINT: str = "SharePoint"
    GOOGLE_DRIVE: str = "GoogleDrive"
    LOCAL_CHUNKS: str = "LocalChunks"


class DeduplicationStrategy(str, Enum):
    OVERWRITE = "Overwrite"
    FAIL = "Fail"


class ChunkingStrategy(str, Enum):
    CHARACTER = "character"


class ArtifactSource(str, Enum):
    S3 = "S3"
    CONFLUENCE = "Confluence"
    SHAREPOINT = "SharePoint"
    GOOGLE_DRIVE = "GoogleDrive"
    LOCAL_FILE = "LocalFile"
    LOCAL_CHUNKS = "LocalChunks"


class StatusType(str, Enum):
    PENDING = "Pending"
    RUNNING = "Running"
    COMPLETED = "Completed"
    FAILED = "Failed"
    CANCELED = "Canceled"
    CHUNKING = "Chunking"
    DELETING = "Deleting"
    UPLOADING = "Uploading"


class UploadJobStatus(str, Enum):
    RUNNING = StatusType.RUNNING.value
    COMPLETED = StatusType.COMPLETED.value
    FAILED = StatusType.FAILED.value
    CANCELED = StatusType.CANCELED.value


class ChunkUploadStatus(str, Enum):
    PENDING = StatusType.PENDING.value
    COMPLETED = StatusType.COMPLETED.value
    FAILED = StatusType.FAILED.value


class CrossEncoderModelName(str, Enum):
    CROSS_ENCODER_MS_MARCO_MINILM_L12_V2 = "cross-encoder/ms-marco-MiniLM-L-12-v2"
    CROSS_ENCODER_MMARCO_MMINILMV2_L12_H384_V1 = "cross-encoder/mmarco-mMiniLMv2-L12-H384-v1"



# copied from  from packages/egp-api-backend/egp_api_backend/server/api/models/egp_models.py
from typing import Optional

from pydantic import BaseModel, Extra

from .parameter_schema import ParameterBindings


class BaseModelRequest(BaseModel):
    model_request_parameters: Optional[ParameterBindings]

    class Config:
        extra = Extra.forbid


class BaseModelResponse(BaseModel):
    class Config:
        extra = Extra.forbid



from typing import Any, List, Optional, Dict

import httpx
from pydantic import BaseModel

from scale_egp.utils.model_utils import dict_without_none_values, make_partial_model
from scale_egp.sdk.models.model_api_models import ModelAlias, ModelAliasRequest
from scale_egp.sdk.models.model_schemas.model_schemas import MODEL_SCHEMAS
from scale_egp.sdk.models.model_schemas.common import BaseModelRequest, BaseModelResponse

from scale_egp.utils.api_utils import APIEngine


# Rename ModelAlias to Model in the SDk
class Model(ModelAlias):
    pass


PartialModelAliasRequest = make_partial_model(ModelAliasRequest)


class ModelCollection(APIEngine):
    """
    Collections class for EGP Models.
    """

    _sub_path = "v3/models"

    def create(
        self,
        name: str,
        model_template_id: str,
        base_model_id: Optional[str] = None,
        model_creation_parameters: Optional[Dict[str, Any]] = None,
        account_id: Optional[str] = None,
    ) -> ModelAlias:
        """
        Create a new EGP Model.
        """
        response = self._post(
            sub_path=self._sub_path,
            request=ModelAliasRequest(
                name=name,
                model_template_id=model_template_id,
                base_model_id=base_model_id,
                model_creation_parameters=model_creation_parameters,
                account_id=account_id or self._api_client.account_id,
            ),
        )
        return Model.from_dict(response.json())

    def get(
        self,
        id: str,
    ) -> Model:
        """
        Get a Model by ID.
        """
        response = self._get(
            sub_path=f"{self._sub_path}/{id}",
        )
        return Model.from_dict(response.json())

    def execute(self, id: str, request: BaseModelRequest) -> BaseModelResponse:
        """
        Get a Model by ID.
        """
        # TODO: verify model_request_parameters matches model template's model_request_parameters_schema if set
        model_alias = self.get(id)
        model_template = self._api_client.model_templates().get(model_alias.model_template_id)
        model_type = model_template.model_type
        model_request_cls, model_response_cls = MODEL_SCHEMAS[model_type]
        assert isinstance(request, model_request_cls)
        response = self._post(
            sub_path=f"{self._sub_path}/{id}/execute",
            request=request,
            timeout=10 * 60,  # 10 minutes
        )
        return model_response_cls(**response.json())

    def update(
        self,
        id: str,
        *,
        name: Optional[str] = None,
        model_template_id: Optional[str] = None,
        base_model_id: Optional[str] = None,
        model_creation_parameters: Optional[dict[str, Any]] = None,
    ) -> Model:
        """
        Update a Model by ID.

        """
        response = self._patch(
            sub_path=f"{self._sub_path}/{id}",
            request=PartialModelAliasRequest(
                **dict_without_none_values(
                    dict(
                        name=name,
                        model_template_id=model_template_id,
                        base_model_id=base_model_id,
                        model_creation_parameters=model_creation_parameters,
                    ),
                )
            ),
        )
        return Model.from_dict(response.json())

    def delete(
        self,
        id: str,
    ) -> bool:
        """
        Delete a Studio Project by ID.

        """
        response = self._delete(
            sub_path=f"{self._sub_path}/{id}",
        )
        return response.status_code == httpx.codes.ok

    def list(
        self,
    ) -> List[Model]:
        """
        List all Studio Projects.

        Returns:
            A list of Studio Projects.
        """
        response = self._get(
            sub_path=self._sub_path,
        )
        return [Model.from_dict(model) for model in response.json()]



from typing import Annotated, Literal, Optional, Union

from pydantic import BaseModel, Field


class ToolRequest(BaseModel):
    name: str  # Name of the tool
    arguments: str  # Arguments (JSON serializable string) to pass to the tool


class AgentMessage(BaseModel):
    role: Literal["agent"] = "agent"
    content: Optional[str] = None  # Output of the agent if finished
    tool_request: Optional[ToolRequest] = None  # Request to run a tool


class ToolMessage(BaseModel):
    role: Literal["tool"] = "tool"
    name: str  # Name of the tool
    content: str  # Output of calling the tool (JSON serialized to string)


class UserMessage(BaseModel):
    role: Literal["user"] = "user"
    content: str


class AssistantMessage(BaseModel):
    role: Literal["assistant"] = "assistant"
    content: str


class SystemMessage(BaseModel):
    role: Literal["system"] = "system"
    content: str


Message = Annotated[
    Union[UserMessage, ToolMessage, AgentMessage, SystemMessage], Field(discriminator="role")
]



from enum import Enum
from typing import List, Optional

from pydantic import BaseModel, Field


class RougeType(str, Enum):
    rouge1 = "rouge1"
    rouge2 = "rouge2"
    rougeL = "rougeL"


class RougeScore(str, Enum):
    precision = "precision"
    recall = "recall"
    fmeasure = "fmeasure"


class RerankRequest(BaseModel):
    query: str = Field(...)
    chunks: List[str] = Field(...)


class CrossEncoderRerankRequest(RerankRequest):
    pass


# Any custom input classes' members MUST be OPTIONAL, as currently the gateway doesn't support anything else
class RougeRerankRequest(RerankRequest):
    rouge_type: Optional[RougeType] = Field(RougeType.rouge2)
    score: Optional[RougeScore] = Field(RougeScore.recall)
    use_stemmer: Optional[bool] = Field(True)
    split_summaries: Optional[bool] = Field(False)


class RerankResponse(BaseModel):
    chunk_scores: List[float]



from typing import List
from pydantic import BaseModel
from httpx import Client
from scale_egp.utils.api_utils import log_curl_command as log_curl


class UserRole(BaseModel):
    role: str
    account_id: str


class UserInfoResponse(BaseModel):
    id: str
    first_name: str
    last_name: str
    email: str
    accounts: List[UserRole]


def get_user_info(
    httpx_client: Client, endpoint_url: str, log_curl_command: bool = False
) -> UserInfoResponse:
    full_url = f"{endpoint_url}user-info"
    if log_curl_command:
        log_curl(httpx_client, "GET", full_url)
    response = httpx_client.get(full_url)
    return UserInfoResponse.parse_obj(response.json())



from typing import Any, Optional, List, Dict

import httpx

from scale_egp.sdk.models import (
    Evaluation,
    EvaluationRequest,
    TestCaseResult,
    TestCaseResultRequest,
    EvaluationConfig,
    TestCase,
    EvaluationDataset,
    TestCaseResultData,
    ApplicationSpec,
)
from scale_egp.utils.api_utils import (
    APIEngine,
)


class EvaluationCollection(APIEngine):
    _sub_path = "v2/evaluations"

    # START sub collections
    def test_case_results(self) -> "TestCaseResultCollection":
        """
        Returns a TestCaseResultCollection for test case results associated with this evaluation.
        """
        return TestCaseResultCollection(self._api_client)

    def create(
        self,
        name: str,
        description: str,
        application_spec: ApplicationSpec,
        evaluation_config: EvaluationConfig,
        tags: Optional[Dict[str, Any]] = None,
        account_id: Optional[str] = None,
    ) -> Evaluation:
        """
        Create a new Evaluation.

        Args:
            name: The name of the Evaluation.
            description: The description of the Evaluation.
            application_spec: The Application Spec to associate the Evaluation with.
            evaluation_config: The configuration for the Evaluation.
            tags: Optional key, value pairs to associate with the Evaluation.

        Returns:
            The newly created Evaluation.
        """
        response = self._post(
            sub_path=self._sub_path,
            request=EvaluationRequest(
                name=name,
                description=description,
                application_spec_id=application_spec.id,
                evaluation_config=evaluation_config,
                tags=tags,
                account_id=account_id or self._api_client.account_id,
            ),
        )
        return Evaluation.from_dict(response.json())

    def get(
        self,
        id: str,
    ) -> Evaluation:
        """
        Get an Evaluation by ID.

        Args:
            id: The ID of the Evaluation.

        Returns:
            The Evaluation.
        """
        response = self._get(
            sub_path=f"{self._sub_path}/{id}",
        )
        return Evaluation.from_dict(response.json())

    def update(
        self,
        id: str,
        name: Optional[str] = None,
        description: Optional[str] = None,
        evaluation_config: Optional[EvaluationConfig] = None,
        tags: Optional[Dict[str, Any]] = None,
    ) -> Evaluation:
        """
        Update an Evaluation.

        Args:
            id: The ID of the Evaluation.
            name: The name of the Evaluation.
            description: The description of the Evaluation.
            evaluation_config: The configuration for the Evaluation.
            tags: Optional key, value pairs to associate with the Evaluation.

        Returns:
            The updated Evaluation.
        """
        response = self._patch(
            sub_path=f"{self._sub_path}/{id}",
            request=EvaluationRequest.partial(
                name=name,
                description=description,
                evaluation_config=evaluation_config,
                tags=tags,
            ),
        )
        return Evaluation.from_dict(response.json())

    def delete(
        self,
        id: str,
    ) -> bool:
        """
        Delete an Evaluation.

        Args:
            id: The ID of the Evaluation.
        """
        response = self._delete(
            sub_path=f"{self._sub_path}/{id}",
        )
        return response.status_code == httpx.codes.ok

    def list(
        self,
    ) -> List[Evaluation]:
        """
        List Evaluations.

        Returns:
            A list of Evaluations.
        """
        response = self._get(
            sub_path=self._sub_path,
        )
        return [Evaluation.from_dict(evaluation) for evaluation in response.json()]


class TestCaseResultCollection(APIEngine):
    _sub_path = "v2/evaluations/{evaluation_id}/test-case-results"

    def create(
        self,
        evaluation: Evaluation,
        evaluation_dataset: EvaluationDataset,
        test_case: TestCase,
        test_case_evaluation_data: TestCaseResultData,
    ) -> TestCaseResult:
        """
        Create a new TestCaseResult.

        Args:
            evaluation: The Evaluation to associate the TestCaseResult with.
            evaluation_dataset: The EvaluationDataset to associate the TestCaseResult with.
            test_case: The TestCase to associate the TestCaseResult with.
            test_case_evaluation_data: The data generated by the application that needs to be
                evaluated.

        Returns:
            The newly created TestCaseResult.
        """
        response = self._post(
            sub_path=self._sub_path.format(evaluation_id=evaluation.id),
            request=TestCaseResultRequest(
                application_spec_id=evaluation.application_spec_id,
                evaluation_dataset_version_num=evaluation_dataset.version_num,
                test_case_id=test_case.id,
                test_case_evaluation_data_schema=test_case.schema_type,
                test_case_evaluation_data=test_case_evaluation_data,
                account_id=self._api_client.account_id,
            ),
        )
        return TestCaseResult.from_dict(response.json())

    def get(
        self,
        id: str,
        evaluation: Evaluation,
    ) -> TestCaseResult:
        """
        Get a TestCaseResult by ID.

        Args:
            id: The ID of the TestCaseResult.
            evaluation: The Evaluation to associate the TestCaseResult with.

        Returns:
            The TestCaseResult.
        """
        response = self._get(
            sub_path=f"{self._sub_path.format(evaluation_id=evaluation.id)}/{id}",
        )
        return TestCaseResult.from_dict(response.json())

    def update(
        self,
        id: str,
        evaluation: Evaluation,
        result: Optional[Dict[str, Any]] = None,
    ) -> TestCaseResult:
        """
        Update a TestCaseResult.

        Args:
            id: The ID of the TestCaseResult.
            evaluation: The Evaluation to associate the TestCaseResult with.
            result: Optional result to associate with the TestCaseResult.

        Returns:
            The updated TestCaseResult.
        """
        response = self._patch(
            sub_path=f"{self._sub_path.format(evaluation_id=evaluation.id)}/{id}",
            request=TestCaseResultRequest.partial(
                result=result,
            ),
        )
        return TestCaseResult.from_dict(response.json())

    def delete(
        self,
        id: str,
        evaluation: Evaluation,
    ) -> bool:
        """
        Delete a TestCaseResult.

        Args:
            id: The ID of the TestCaseResult.
            evaluation: The Evaluation to associate the TestCaseResult with.
        """
        response = self._delete(
            sub_path=f"{self._sub_path.format(evaluation_id=evaluation.id)}/{id}",
        )
        return response.status_code == httpx.codes.ok

    def list(
        self,
        evaluation: Evaluation,
    ) -> List[TestCaseResult]:
        """
        List TestCaseResults.

        Args:
            evaluation: The Evaluation to associate the TestCaseResults with.

        Returns:
            A list of TestCaseResults.
        """
        response = self._get(
            sub_path=self._sub_path.format(evaluation_id=evaluation.id),
        )
        return [TestCaseResult.from_dict(test_case_result) for test_case_result in response.json()]



# these pydantic model definitions were copied from
# packages/egp-api-backend/egp_api_backend/server/internal/entities.py
from typing import List

from .completions_shared import CompletionBaseRequest, CompletionBaseResponse
from .message import Message


class ChatCompletionRequest(CompletionBaseRequest):
    messages: List[Message]


class ChatCompletionResponse(CompletionBaseResponse):
    message: Message



from typing import List

from .common import BaseModelRequest, BaseModelResponse


class RerankingRequest(BaseModelRequest):
    query: str
    chunks: List[str]


class RerankingResponse(BaseModelResponse):
    chunk_scores: List[float]
